@@ -6,32 +6,105 @@
 //#include "slow5.h"
 #include "slow5_extra.h"
 #include "slow5_misc.h"
+#include "slow5_byte.h"
 //#include "slow5_error.h"
 
 extern enum slow5_log_level_opt  slow5_log_level;
 extern enum slow5_exit_condition_opt  slow5_exit_condition;
+extern int8_t slow5_bigend;
+extern int8_t slow5_skip_rid;
 
 #define BUF_INIT_CAP (20*1024*1024)
 #define SLOW5_INDEX_BUF_INIT_CAP (64) // 2^6 TODO is this too little?
 
-static inline struct slow5_idx *slow5_idx_init_empty(void);
-static int slow5_idx_build(struct slow5_idx *index, struct slow5_file *s5p);
-static int slow5_idx_read(struct slow5_idx *index);
 
-static inline struct slow5_idx *slow5_idx_init_empty(void) {
+static int slow5_idx_build(struct slow5_idx *index, struct slow5_file *s5p);
+int slow5_idx_read(struct slow5_idx *index);
+
+struct slow5_idx *slow5_idx_init_empty(void) {
 
     struct slow5_idx *index = (struct slow5_idx *) calloc(1, sizeof *index);
-    SLOW5_MALLOC_CHK(index);
+    if(!index){
+        SLOW5_MALLOC_ERROR();
+        slow5_errno = SLOW5_ERR_MEM;
+        return NULL;
+    }
     index->hash = kh_init(slow5_s2i);
 
     return index;
 }
 
+static inline int slow5_idx_load_fp(struct slow5_idx *index, struct slow5_file *s5p, FILE *index_fp, int8_t check_ts) {
+
+    index->fp = index_fp;
+
+    if(check_ts){
+        int err;
+        if (slow5_filestamps_cmp(index->pathname, s5p->meta.pathname, &err) < 0.0) {
+            SLOW5_WARNING("Index file '%s' is older than slow5 file '%s'.",
+                    index->pathname, s5p->meta.pathname);
+        }
+        if (err == -1) {
+            return -1;
+        }
+    } else {
+        SLOW5_INFO("Custom index file '%s' is being used. Time stamps not checked.", index->pathname);
+    }
+
+    if (slow5_idx_read(index) != 0) {
+        return -1;
+    }
+    if (index->version.major != s5p->header->version.major ||
+            index->version.minor != s5p->header->version.minor ||
+            index->version.patch != s5p->header->version.patch) {
+        SLOW5_ERROR("Index file version '" SLOW5_VERSION_STRING_FORMAT "' is different to slow5 file version '" SLOW5_VERSION_STRING_FORMAT "'. Please re-index.",
+                index->version.major, index->version.minor, index->version.patch,
+                s5p->header->version.major, s5p->header->version.minor, s5p->header->version.patch);
+        return -1;
+    }
+
+    return 0;
+}
+
+struct slow5_idx *slow5_idx_init_with(struct slow5_file *s5p, const char *pathname) {
+
+    struct slow5_idx *index = slow5_idx_init_empty();
+    if (!index) {
+        SLOW5_MALLOC_ERROR();
+        slow5_errno = SLOW5_ERR_MEM;
+        return NULL;
+    }
+    index->pathname = strdup(pathname);
+    if (!index->pathname) {
+        slow5_idx_free(index);
+        return NULL;
+    }
+
+    FILE *index_fp;
+
+    if ((index_fp = fopen(index->pathname, "r")) == NULL) { // If file doesn't exist
+        SLOW5_ERROR("Index file not found at '%s'.", index->pathname)
+        slow5_errno =  SLOW5_ERR_NOIDX;
+        slow5_idx_free(index);
+        return NULL;
+    } else {
+        if (slow5_idx_load_fp(index, s5p, index_fp, 0) != 0) {
+            slow5_idx_free(index);
+            return NULL;
+        }
+    }
+
+    return index;
+}
+
+
 // TODO return NULL if idx_init fails
 struct slow5_idx *slow5_idx_init(struct slow5_file *s5p) {
 
     struct slow5_idx *index = slow5_idx_init_empty();
     if (!index) {
+        SLOW5_MALLOC_ERROR();
+        slow5_errno = SLOW5_ERR_MEM;
         return NULL;
     }
     index->pathname = slow5_get_idx_path(s5p->meta.pathname);
@@ -50,6 +123,12 @@ struct slow5_idx *slow5_idx_init(struct slow5_file *s5p) {
             return NULL;
         }
         index->fp = fopen(index->pathname, "w");
+        if(!index->fp){
+            SLOW5_ERROR("Error opening file '%s' for writing: %s.", index->pathname, strerror(errno));
+            slow5_errno = SLOW5_ERR_IO;
+            slow5_idx_free(index);
+            return NULL;
+        }
         if (slow5_idx_write(index, s5p->header->version) != 0) {
             slow5_idx_free(index);
             return NULL;
@@ -57,26 +136,7 @@ struct slow5_idx *slow5_idx_init(struct slow5_file *s5p) {
         fclose(index->fp);
         index->fp = NULL;
     } else {
-        index->fp = index_fp;
-        int err;
-        if (slow5_filestamps_cmp(index->pathname, s5p->meta.pathname, &err) < 0.0) {
-            SLOW5_WARNING("Index file '%s' is older than slow5 file '%s'.",
-                    index->pathname, s5p->meta.pathname);
-        }
-        if (err == -1) {
-            slow5_idx_free(index);
-            return NULL;
-        }
-        if (slow5_idx_read(index) != 0) {
-            slow5_idx_free(index);
-            return NULL;
-        }
-        if (index->version.major != s5p->header->version.major ||
-                index->version.minor != s5p->header->version.minor ||
-                index->version.patch != s5p->header->version.patch) {
-            SLOW5_ERROR("Index file version '" SLOW5_VERSION_STRING_FORMAT "' is different to slow5 file version '" SLOW5_VERSION_STRING_FORMAT "'. Please re-index.",
-                    index->version.major, index->version.minor, index->version.patch,
-                    s5p->header->version.major, s5p->header->version.minor, s5p->header->version.patch);
+        if (slow5_idx_load_fp(index, s5p, index_fp, 1) != 0) {
             slow5_idx_free(index);
             return NULL;
         }
@@ -102,6 +162,13 @@ int slow5_idx_to(struct slow5_file *s5p, const char *pathname) {
     }
 
     index->fp = fopen(pathname, "w");
+    if(!index->fp){
+        SLOW5_ERROR("Error opening file '%s' for writing: %s.", pathname, strerror(errno));
+        slow5_errno = SLOW5_ERR_IO;
+        slow5_idx_free(index);
+        return -1;
+    }
+
     if (slow5_idx_write(index, s5p->header->version) != 0) {
         slow5_idx_free(index);
         return -1;
@@ -119,7 +186,15 @@ int slow5_idx_to(struct slow5_file *s5p, const char *pathname) {
 static int slow5_idx_build(struct slow5_idx *index, struct slow5_file *s5p) {
 
     uint64_t curr_offset = ftello(s5p->fp);
+    if(curr_offset == -1){
+        SLOW5_ERROR("Could not ftell SLOW5 file: %s.", strerror(errno));
+        slow5_errno = SLOW5_ERR_IO;
+        return -1;
+    }
+
     if (fseeko(s5p->fp, s5p->meta.start_rec_offset, SEEK_SET != 0)) {
+        SLOW5_ERROR("Could not fseek SLOW5 file: %s.", strerror(errno));
+        slow5_errno = SLOW5_ERR_IO;
         return -1;
     }
 
@@ -129,11 +204,21 @@ static int slow5_idx_build(struct slow5_idx *index, struct slow5_file *s5p) {
     if (s5p->format == SLOW5_FORMAT_ASCII) {
         size_t cap = BUF_INIT_CAP;
         char *buf = (char *) malloc(cap * sizeof *buf);
-        SLOW5_MALLOC_CHK(buf);
+        if(!buf){
+            SLOW5_MALLOC_ERROR();
+            slow5_errno = SLOW5_ERR_MEM;
+            return -1;
+        }
         ssize_t buf_len;
         char *bufp;
 
         offset = ftello(s5p->fp);
+        if(offset == -1){
+            SLOW5_ERROR("Could not ftell SLOW5 file: %s.", strerror(errno));
+            slow5_errno = SLOW5_ERR_IO;
+            return -1;
+        }
+
         while ((buf_len = getline(&buf, &cap, s5p->fp)) != -1) { // TODO this return is closer int64_t not unsigned
             bufp = buf;
             char *read_id = strdup(slow5_strsep(&bufp, SLOW5_SEP_COL));
@@ -154,6 +239,11 @@ static int slow5_idx_build(struct slow5_idx *index, struct slow5_file *s5p) {
         while (1) {
             // Set start offset
             offset = ftello(s5p->fp);
+            if(offset == -1){
+                SLOW5_ERROR("Could not ftell SLOW5 file: %s.", strerror(errno));
+                slow5_errno = SLOW5_ERR_IO;
+                return -1;
+            }
 
             // Get record size
             slow5_rec_size_t record_size;
@@ -183,6 +273,7 @@ static int slow5_idx_build(struct slow5_idx *index, struct slow5_file *s5p) {
                 }
                 return slow5_errno;
             }
+            SLOW5_BYTE_SWAP(&record_size); //if bigendian
 
             size = sizeof record_size + record_size;
 
@@ -301,10 +392,16 @@ int slow5_idx_write(struct slow5_idx *index, struct slow5_version version) {
         struct slow5_rec_idx read_index = kh_value(index->hash, pos);
 
         slow5_rid_len_t read_id_len = strlen(index->ids[i]);
-        if (fwrite(&read_id_len, sizeof read_id_len, 1, index->fp) != 1 ||
-                fwrite(index->ids[i], sizeof *index->ids[i], read_id_len, index->fp) != read_id_len ||
-                fwrite(&read_index.offset, sizeof read_index.offset, 1, index->fp) != 1 ||
-                fwrite(&read_index.size, sizeof read_index.size, 1, index->fp) != 1) {
+        if (SLOW5_FWRITE(&read_id_len, sizeof read_id_len, 1, index->fp) != 1){
+            return SLOW5_ERR_IO;
+        }
+        if (fwrite(index->ids[i], sizeof *index->ids[i], read_id_len, index->fp) != read_id_len){
+            return SLOW5_ERR_IO;
+        }
+        if (SLOW5_FWRITE(&read_index.offset, sizeof read_index.offset, 1, index->fp) != 1 ){
+            return SLOW5_ERR_IO;
+        }
+        if (SLOW5_FWRITE(&read_index.size, sizeof read_index.size, 1, index->fp) != 1) {
             return SLOW5_ERR_IO;
         }
     }
@@ -317,7 +414,7 @@ int slow5_idx_write(struct slow5_idx *index, struct slow5_version version) {
     return 0;
 }
 
-static int slow5_idx_read(struct slow5_idx *index) {
+int slow5_idx_read(struct slow5_idx *index) {
 
     struct slow5_version max_supported = SLOW5_VERSION_ARRAY;
     const char magic[] = SLOW5_INDEX_MAGIC_NUMBER;
@@ -347,7 +444,7 @@ static int slow5_idx_read(struct slow5_idx *index) {
 
     while (1) {
         slow5_rid_len_t read_id_len;
-        if (fread(&read_id_len, sizeof read_id_len, 1, index->fp) != 1) {
+        if (SLOW5_FREAD(&read_id_len, sizeof read_id_len, 1, index->fp) != 1) {
             SLOW5_ERROR("Malformed slow5 index. Failed to read the read ID length.%s", feof(index->fp) ? " Missing index end of file marker." : "");
             if (feof(index->fp)) {
                 slow5_errno = SLOW5_ERR_TRUNC;
@@ -385,8 +482,8 @@ static int slow5_idx_read(struct slow5_idx *index) {
         uint64_t offset;
         uint64_t size;
 
-        if (fread(&offset, sizeof offset, 1, index->fp) != 1 ||
-                fread(&size, sizeof size, 1, index->fp) != 1) {
+        if (SLOW5_FREAD(&offset, sizeof offset, 1, index->fp) != 1 ||
+                SLOW5_FREAD(&size, sizeof size, 1, index->fp) != 1) {
             return SLOW5_ERR_IO;
         }
 
@@ -439,7 +536,9 @@ int slow5_idx_get(struct slow5_idx *index, const char *read_id, struct slow5_rec
 
     khint_t pos = kh_get(slow5_s2i, index->hash, read_id);
     if (pos == kh_end(index->hash)) {
-        SLOW5_ERROR("Read ID '%s' was not found.", read_id)
+        if (slow5_skip_rid == 0) {
+            SLOW5_ERROR("Read ID '%s' was not found.", read_id)
+        }
         ret = -1;
     } else if (read_index) {
         *read_index = kh_value(index->hash, pos);
